import threading
import Queue
import time

_SLEEP_TIME = 0.01

class LoopingThread(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.daemon = True
        self.stopFlag = threading.Event()

    def stop(self):
        self.stopFlag.set()

    def init(self):
        pass

    def worker(self):
        pass

    def cleanup(self):
        pass

    def run(self):
        self.init()
        self.stopFlag.clear()
        while not self.stopFlag.isSet():
            self.worker()
        self.stop()
        self.cleanup()

class _BlockingResponse:
    def __init__(self):
        self.data = None
        self.recvFlag = threading.Event()
        self.recvFlag.clear()
        
    def callback(self, data):
        self.data = data
        self.recvFlag.set()

    def getData(self, timeout = None):
        if timeout:
            endTime = time.time() + timeout
        while not timeout or time.time() < endTime:
            if self.recvFlag.isSet():
                break
            time.sleep(_SLEEP_TIME)
        return self.data

class QueueThread(LoopingThread):
    def __init__(self):
        LoopingThread.__init__(self)
        self.seqn = 0
        self.queue = Queue.Queue()
        self.callbacksLock = threading.Lock()
        self.callbacks = {}

    def request(self, data, callback = None, block = False, timeout = None):
        resp = None
        if block:
            resp = _BlockingResponse()
            callback = resp.callback

        self.callbacksLock.acquire()
        self.callbacks[self.seqn] = callback
        self.callbacksLock.release()
        self.queue.put((self.seqn,data))
        self.seqn = self.seqn + 1

        if block:
            return resp.getData(timeout)

    def worker(self):
        try:
            seqn, data = self.queue.get(block=False)
        except Queue.Empty as e:
            time.sleep(_SLEEP_TIME)
            return
        
        callback = None
        self.callbacksLock.acquire()
        if seqn in self.callbacks:
            callback = self.callbacks[seqn]
        self.callbacksLock.release()
                            
        if callback:
            callback(data)
            self.callbacksLock.acquire()
            self.callbacks[seqn] = None
            self.callbacksLock.release()
        
